package speiger.src.coreengine;

import java.util.List;

import org.lwjgl.glfw.GLFW;
import org.lwjgl.opengl.GL;
import org.lwjgl.opengl.GL11;

import com.google.gson.JsonObject;

import speiger.src.coreengine.assets.AssetLocation;
import speiger.src.coreengine.assets.AssetManager;
import speiger.src.coreengine.assets.base.IAsset;
import speiger.src.coreengine.assets.base.IAssetPackage;
import speiger.src.coreengine.assets.base.IAssetProvider;
import speiger.src.coreengine.rendering.gui.font.glyth.Glyth;
import speiger.src.coreengine.rendering.gui.font.glyth.UnbakedGlyth;
import speiger.src.coreengine.rendering.gui.font.glyth.UnbakedGlyth.GlythBaker;
import speiger.src.coreengine.rendering.gui.font.glyth.IGlythSheetInfo;
import speiger.src.coreengine.rendering.gui.font.providers.IFontProvider;
import speiger.src.coreengine.rendering.gui.font.providers.STBTrueTypeProvider;
import speiger.src.coreengine.rendering.guiOld.renderer.provider.BitmapFontProvider;
import speiger.src.coreengine.rendering.input.devices.FileDrop;
import speiger.src.coreengine.rendering.input.devices.Joystick;
import speiger.src.coreengine.rendering.input.devices.Keyboard;
import speiger.src.coreengine.rendering.input.devices.Mouse;
import speiger.src.coreengine.rendering.input.events.KeyEvent;
import speiger.src.coreengine.rendering.input.window.Window;
import speiger.src.coreengine.rendering.input.window.WindowManager;
import speiger.src.coreengine.rendering.models.buffers.BufferAttribute;
import speiger.src.coreengine.rendering.models.buffers.VertexArray;
import speiger.src.coreengine.rendering.shader.Shader;
import speiger.src.coreengine.rendering.shader.SimpleShader;
import speiger.src.coreengine.rendering.shader.uniform.base.TextureUniform;
import speiger.src.coreengine.rendering.tesselation.buffer.VertexBuilder;
import speiger.src.coreengine.rendering.tesselation.format.VertexTypes;
import speiger.src.coreengine.rendering.textures.custom.Drawable;
import speiger.src.coreengine.rendering.textures.custom.DynamicTexture;
import speiger.src.coreengine.rendering.utils.GLStateTracker;
import speiger.src.coreengine.rendering.utils.values.GLDataType;
import speiger.src.coreengine.rendering.utils.values.GLMode;
import speiger.src.coreengine.rendering.utils.values.textures.GLTextureFormat;
import speiger.src.coreengine.utils.eventbus.EventBus;
import speiger.src.coreengine.utils.helpers.IOUtils;

public class NewInputTest {
	
	EventBus bus = new EventBus();
	WindowManager manager = new WindowManager();
	AssetManager assets = new AssetManager(List.of(IAssetPackage.of(IOUtils.getBaseLocation())));
	private Shader<TestShader> shaderTest = Shader.create(TestShader::new);
	
	
	public static void main(String[] args) {
		new NewInputTest().run();
	}
	
	public void run() {
		GLFW.glfwInit();
		manager.initialize();
		Mouse.INSTANCE.init(bus);
		Keyboard.INSTANCE.init(bus);
		Joystick.INSTANCE.init(manager, bus);
		FileDrop.INSTANCE.init(bus);
		manager.addDevices(Mouse.INSTANCE, Keyboard.INSTANCE, Joystick.INSTANCE, FileDrop.INSTANCE);
		Window window = manager.builder().title("Testing Engine").build();
		shaderTest.register();
		assets.addListener(GLStateTracker.instance().shaders);
		assets.addListener(GLStateTracker.TEXTURE_TRACKER);
		assets.reload();
		
		System.out.println("Testing: "+GL.getCapabilities().OpenGL46);
		System.out.println("Testing: "+Integer.divideUnsigned(-1, 255));
		
		IFontProvider provider = STBTrueTypeProvider.create(AssetLocation.of("font/roboto/font.json"), assets);
		BitmapFontProvider bit = loadProvider(AssetLocation.of("font/roboto.json"), 18.5F);
		
		int size = 512;
		int half = size >> 1;
		
		DynamicTexture texture = new DynamicTexture(size, size, DynamicTexture.DEFAULT_PARAMETERS);
		texture.fill(0, 0, size, size, -1);
		texture.fill(0, 0, half, half, 255, 0, 0, 255);
		texture.fill(half, 0, half, half, 0, 255, 0, 255);
		texture.fill(half, half, half, half, 255, 0, 0, 255);
		texture.fill(0, half, half, half, 0, 0, 255, 255);
		texture.process(true);
		window.visible(true);
		
		bus.register(KeyEvent.Key.class, T -> {
			if(T.key() == GLFW.GLFW_KEY_T) {
				T.cancel();
				texture.bind();
				Drawable drawable = new Drawable(GLTextureFormat.RGBA, half, half);
				drawable.fill(0, 0, half, half, 255, 255, 0, 255);
				drawable.upload(192, 192, 64, 64, 128, 128);
				drawable.close();
			}
			else if(T.key() == GLFW.GLFW_KEY_Z) {
				T.cancel();
				texture.bind();
				UnbakedGlyth data = provider.glythData("C".codePointAt(0), 0);
				data.bake(new GlythBaker() {
					@Override
					public Glyth bake(IGlythSheetInfo info) {
						int width = info.width();
						int height = info.height();
						info.upload(half - (width >> 2), half - (height >> 2));
						return null;
					}
				}, 1F, 1F);
			}
		});
		
		VertexBuilder builder = new VertexBuilder(255);
		builder.start(GLMode.TRIANGLES, VertexTypes.TESTING);
		builder.pos(-0.5F, -0.5F, 0).tex(0F, 1F).rgba(-1).endVertex();
		builder.pos(0.5F, -0.5F, 0).tex(1F, 1F).rgba(-1).endVertex();
		builder.pos(0.5F, 0.5F, 0).tex(1F, 0F).rgba(-1).endVertex();
		builder.pos(0.5F, 0.5F, 0).tex(1F, 0F).rgba(-1).endVertex();
		builder.pos(-0.5F, 0.5F, 0).tex(0F, 0F).rgba(-1).endVertex();
		builder.pos(-0.5F, -0.5F, 0).tex(0F, 1F).rgba(-1).endVertex();
		
		TestModel model = new TestModel(builder.getBytes());
		
		GL11.glClearColor(0.2F, 0.55F, 0.66F, 1F);
		while(!window.shouldClose()) {
			GLFW.glfwPollEvents();
			window.beginFrame();
			GL11.glClear(GL11.GL_COLOR_BUFFER_BIT | GL11.GL_DEPTH_BUFFER_BIT);
			shaderTest.bind();
			shaderTest.get().texture.set(texture);
			model.bindArray();
			GLStateTracker.drawArrays(GLMode.TRIANGLES.glValue(), 6);
			
			model.unbindArray();
			window.handleInput();
			window.finishFrame();
			try { Thread.sleep(100); }
			catch(InterruptedException e) { e.printStackTrace(); }
		}
		window.destroy();
		manager.destroy();
	}
	
	private BitmapFontProvider loadProvider(AssetLocation location, float desiredSize)
	{
		try(IAsset asset = assets.getAsset(location))
		{
			JsonObject obj = asset.json();
			return BitmapFontProvider.load(obj, desiredSize, null);
		}
		catch(Exception e) { e.printStackTrace(); }
		return null;
	}
	
	public static class TestShader extends SimpleShader {
		public TextureUniform texture = uniforms.addTexture("texture", 0);
		public TestShader(IAssetProvider provider) {
			super(provider, "testing_shader", AssetLocation.of("shader/testing/vertex.vs"), AssetLocation.of("shader/testing/fragment.fs"), "in_position", "in_tex", "in_color");
		}
	}
	
	public static class TestModel extends VertexArray {
		public TestModel(byte[] data) {
			bind();
			createBuffer(new BufferAttribute(0, 3), new BufferAttribute(1, 2), new BufferAttribute(2, 4, GLDataType.UNSIGNED_BYTE)).set(data).unbind();
			unbind();
		}
	}
}
