package speiger.src.coreengine.rendering.gui.font;

import speiger.src.collections.ints.maps.impl.hash.Int2ObjectOpenHashMap;
import speiger.src.collections.ints.maps.interfaces.Int2ObjectMap;
import speiger.src.coreengine.rendering.gui.font.glyth.Glyth;
import speiger.src.coreengine.rendering.gui.font.glyth.GlythData;
import speiger.src.coreengine.rendering.gui.font.glyth.UnbakedGlyth.GlythBaker;

public class Font {
	FontGroup fonts;
	GlythBaker baker;
	float size;
	float oversample;
	GlythCache[] styledCache = new GlythCache[] {new GlythCache(0), new GlythCache(1), new GlythCache(2), new GlythCache(3)};
	
	private Font(FontGroup fonts, GlythBaker baker, float size, float oversample) {
		this.fonts = fonts;
		this.baker = baker;
		this.size = size;
		this.oversample = oversample;
	}
	
	public GlythData data(int codepoint, int style) {
		return styledCache[style & 0x3].data(codepoint);
	}
	
	public Glyth glyth(int codepoint, int style) {
		return styledCache[style & 0x3].glyth(codepoint);
	}
	
	private class GlythCache {
		final int style;
		Int2ObjectMap<GlythData> data = new Int2ObjectOpenHashMap<>();
		Int2ObjectMap<Glyth> glyth = new Int2ObjectOpenHashMap<>();
		
		public GlythCache(int style) {
			this.style = style;
		}

		public GlythData data(int codepoint) {
			return data.computeIfAbsent(codepoint, this::compute);
		}
		
		public Glyth glyth(int codepoint) {
			return glyth.computeIfAbsent(codepoint, this::bake);
		}
		
		private GlythData compute(int codepoint) {
			return new GlythData(fonts.data(codepoint, style), size);
		}
		
		private Glyth bake(int codepoint) {
			return fonts.data(codepoint, style).bake(baker, size, oversample);
		}
	}
}
