package speiger.src.coreengine.rendering.utils;

import org.lwjgl.opengl.GL11;
import org.lwjgl.opengl.GL15;
import org.lwjgl.opengl.GL30;

import speiger.src.coreengine.rendering.textures.base.TextureMetadata;
import speiger.src.coreengine.rendering.utils.values.IGLValue;
import speiger.src.coreengine.rendering.utils.values.IGLValue.IGLDataType;
import speiger.src.coreengine.rendering.utils.values.IGLValue.ITextureFormatType;
import speiger.src.coreengine.rendering.utils.values.IGLValue.ITextureParameter;

public class GLFunctions {
	
	public static int createTexture() {
		return GL11.glGenTextures();
	}
	
	public static void deleteTextures(int... id) {
		GL11.glDeleteTextures(id);
	}
	
	public static void textureParameter(IGLValue type, ITextureParameter parameter, IGLValue value) {
		textureParameter(type, parameter, value.glValue());
	}
	
	public static void textureParameter(IGLValue type, ITextureParameter parameter, int value) {
		if(!parameter.isValid(value)) throw new IllegalArgumentException("Value ["+value+"] isn't supported for GL Function ["+parameter.glValue()+"], only int functions are supported with this");
		textureParameter(type.glValue(), parameter.glValue(), value);
	}
	
	public static void textureParameter(int type, int parameter, int value) {
		GL15.glTexParameteri(type, parameter, value);
	}
	
	public static void upload2DImage(IGLValue target, int mipmaps, TextureMetadata metadata, int width, int height, int border, long pixels) {
		upload2DImage(target.glValue(), mipmaps, metadata.getInternalFormat(), width, width, border, metadata.getExternalFormat(), metadata.getDataFormat(), pixels);
	}
	
	public static void upload2DImage(IGLValue target, int mipmaps, ITextureFormatType internalFormat, int width, int height, int border, ITextureFormatType externalFormat, IGLDataType data, long pixels) {
		upload2DImage(target.glValue(), mipmaps, internalFormat.glValue(), width, height, border, externalFormat.glValue(), data.glValue(), pixels);
	}
	
	public static void upload2DImage(int target, int mipmaps, int internalFormat, int width, int height, int border, int externalFormat, int dataFormat, long pixels) {
		GL11.glTexImage2D(target, mipmaps, internalFormat, width, height, border, externalFormat, dataFormat, pixels);
	}
	
	public static void upload2DSubImage(IGLValue target, int level, int xoffset, int yoffset, int width, int height, ITextureFormatType format, IGLDataType type, long pixels) {
		upload2DSubImage(target.glValue(), level, xoffset, yoffset, width, height, format.glValue(), type.glValue(), pixels);
	}
	
	public static void upload2DSubImage(int target, int level, int xOffset, int yOffset, int width, int height, int format, int type, long pixels) {
		GL15.nglTexSubImage2D(target, level, xOffset, yOffset, width, height, format, type, pixels);

	}
	
	public static void generateMipMaps(IGLValue type) {
		GL30.glGenerateMipmap(type.glValue());
	}
		
	public static void pixelStoreI(int mode, int value) {
		GL11.glPixelStorei(mode, value);
	}
}
